import 'package:flutter/material.dart';

class ScreenUtil {
  static ScreenUtil instance = new ScreenUtil();

  double width;
  double height;
  bool allowFontScaling;

  static MediaQueryData _mediaQueryData;
  static double _screenWidth;
  static double _screenHeight;
  static double _pixelRatio;
  static double _statusBarHeight;
  static double _bottomBarHeight;
  static double _textScaleFactor;

  ScreenUtil({
    this.width = 1080,
    this.height = 1920,
    this.allowFontScaling = false,
  });

  static ScreenUtil getInstance() {
    return instance;
  }

  void init(BuildContext context) {
    MediaQueryData mediaQuery = MediaQuery.of(context);
    _mediaQueryData = mediaQuery;
    _pixelRatio = mediaQuery.devicePixelRatio;
    _screenWidth = mediaQuery.size.width;
    _screenHeight = mediaQuery.size.height;
    _statusBarHeight = mediaQuery.padding.top;
    _bottomBarHeight = _mediaQueryData.padding.bottom;
    _textScaleFactor = mediaQuery.textScaleFactor;
  }

  static MediaQueryData get mediaQueryData => _mediaQueryData;

  //The number of font pixels per logical pixel, the scaling of the font
  static double get textScaleFactory => _textScaleFactor;

  //Device pixel density
  static double get pixelRatio => _pixelRatio;

  //Current device width dp
  static double get screenWidthDp => _screenWidth;

  //Current device height dp
  static double get screenHeightDp => _screenHeight;

  //Current device width px
  static double get screenWidth => _screenWidth * _pixelRatio;

  //Current device height px
  static double get screenHeight => _screenHeight * _pixelRatio;

  //Status bar height in dp
  static double get statusBarHeight => _statusBarHeight;

  //Bottom safe zone distance dp
  static double get bottomBarHeight => _bottomBarHeight;

  //The ratio of the actual dp to the design draft px
  get scaleWidth => _screenWidth / instance.width;

  get scaleHeight => _screenHeight / instance.height;

  /// Adapted to the device width of the design draft
  /// Height is also adapted according to this to ensure no deformation
  setWidth(double width) => width * scaleWidth;

  /// Highly adapted to the design of the device
  /// When it is found that one screen in the design draft does not match the current style effect,
  /// or when there is a difference in shape, it is recommended to use this method for height adaptation.
  /// The height adaptation is mainly for the same effect that you want to display according to the one screen of the design draft.
  setHeight(double height) => height * scaleHeight;

  /// Font size adaptation method
  /// @param fontSize Pass font in px on the design draft,
  /// @param allowFontScaling Control whether the font is based on the system “font size”。by default its false。
  /// @param allowFontScaling Specifies whether fonts should scale to respect Text Size accessibility settings. The default is false.
  setSp(double fontSize) => allowFontScaling
      ? fontSize * _textScaleFactor
      : (fontSize * _textScaleFactor) / _textScaleFactor;
}
